@@ -72,6 +72,61 @@ QString copyAndFree(char *str)
     g_free(str);
     return ret;
 }
+
+class ProgressCollector : public QObject
+{
+    Q_OBJECT
+public:
+    ProgressCollector(FlatpakBackend *backend)
+        : m_backend(backend)
+    {
+        connect(m_backend->m_updater, &StandardBackendUpdater::settingUpChanged, this, &ProgressCollector::progressChanged);
+    }
+
+    void add(FlatpakRefreshAppstreamMetadataJob *job)
+    {
+        m_lastProgress = 0;
+        m_jobs << job;
+        connect(job, &FlatpakRefreshAppstreamMetadataJob::progressChanged, this, &ProgressCollector::progressChanged);
+        connect(job, &FlatpakRefreshAppstreamMetadataJob::finished, this, [this]() {
+            if (std::ranges::all_of(m_jobs, [](FlatpakRefreshAppstreamMetadataJob *j) {
+                    return j->isFinished();
+                })) {
+                if (std::ranges::any_of(m_jobs, [](FlatpakRefreshAppstreamMetadataJob *j) -> bool {
+                        return j->hasChanged();
+                    })) {
+                    m_backend->loadLocalUpdates();
+                }
+                qDeleteAll(m_jobs);
+                m_jobs.clear();
+                Q_EMIT progressChanged();
+            }
+        });
+        Q_EMIT progressChanged();
+    }
+
+    uint progress()
+    {
+        if (!m_jobs.isEmpty()) {
+            uint sum = 0;
+            for (auto job : std::as_const(m_jobs)) {
+                sum += job->progress();
+            }
+            static constexpr uint s_almostPercentage = 95; // We can't give 100% because we still have to process the updates locally and it's not trivial
+            m_lastProgress = std::min<uint>(s_almostPercentage, std::max<uint>(m_lastProgress, sum / m_jobs.count()));
+            return m_lastProgress;
+        }
+        return m_backend->m_updater->isSettingUp() ? m_lastProgress : 100;
+    }
+
+Q_SIGNALS:
+    void progressChanged();
+
+private:
+    uint m_lastProgress = 0;
+    FlatpakBackend *const m_backend;
+    QList<FlatpakRefreshAppstreamMetadataJob *> m_jobs;
+};
 }
 
 class FlatpakSource
@@ -162,17 +217,6 @@ public:
         return m_remote;
     }
 
-    AppStream::ComponentBox componentsByName(const QString &name)
-    {
-        auto components = m_pool->componentsById(name);
-        if (!components.result().isEmpty()) {
-            return components.result();
-        }
-
-        components = m_pool->componentsByProvided(AppStream::Provided::KindId, name);
-        return components.result();
-    }
-
     AppStream::ComponentBox componentsByFlatpakId(const QString &ref)
     {
         auto components = m_pool->componentsByBundleId(AppStream::Bundle::KindFlatpak, ref, false);
@@ -287,11 +331,12 @@ FlatpakBackend::FlatpakBackend(QObject *parent)
     , m_reviews(OdrsReviewsBackend::global())
     , m_cancellable(g_cancellable_new())
     , m_checkForUpdatesTimer(new QTimer(this))
+    , m_collector(new Utils::ProgressCollector(this))
 {
     g_autoptr(GError) error = nullptr;
 
     connect(m_updater, &StandardBackendUpdater::updatesCountChanged, this, &FlatpakBackend::updatesCountChanged);
-    connect(m_updater, &StandardBackendUpdater::settingUpChanged, this, &FlatpakBackend::fetchingUpdatesProgressChanged);
+    connect(m_collector, &Utils::ProgressCollector::progressChanged, this, &FlatpakBackend::fetchingUpdatesProgressChanged);
 
     // Load flatpak installation
     if (!setupFlatpakInstallations(&error)) {
@@ -1054,16 +1099,6 @@ void FlatpakBackend::metadataRefreshed(FlatpakRemote *remote)
 {
     const bool removed = m_refreshAppstreamMetadataJobs.remove(remote);
     Q_ASSERT(removed);
-    if (m_refreshAppstreamMetadataJobs.isEmpty()) {
-        for (auto installation : std::as_const(m_installations)) {
-            // Load local updates, comparing current and latest commit
-            loadLocalUpdates(installation);
-
-            if (g_cancellable_is_cancelled(m_cancellable)) {
-                break;
-            }
-        }
-    }
 }
 
 void FlatpakBackend::createPool(QSharedPointer<FlatpakSource> source)
@@ -1114,17 +1149,17 @@ void FlatpakBackend::createPool(QSharedPointer<FlatpakSource> source)
     pool->loadAsync();
 }
 
-QSharedPointer<FlatpakSource> FlatpakBackend::integrateRemote(FlatpakInstallation *flatpakInstallation, FlatpakRemote *remote)
+QSharedPointer<FlatpakSource> FlatpakBackend::integrateRemote(GLibHolder<FlatpakInstallation> flatpakInstallation, GLibHolder<FlatpakRemote> remote)
 {
-    Q_ASSERT(m_refreshAppstreamMetadataJobs.contains(remote));
-    m_sources->addRemote(remote, flatpakInstallation);
+    Q_ASSERT(m_refreshAppstreamMetadataJobs.contains(remote.get()));
+    m_sources->addRemote(remote.get(), flatpakInstallation.get());
     const auto matchRemote = [this, flatpakInstallation, remote](const auto &sources) -> QSharedPointer<FlatpakSource> {
         for (auto source : sources) {
-            if (source->url() == copyAndFree(flatpak_remote_get_url(remote)) && source->installation() == flatpakInstallation
-                && source->name() == QString::fromUtf8(flatpak_remote_get_name(remote))) {
+            if (source->url() == copyAndFree(flatpak_remote_get_url(remote.get())) && source->installation() == flatpakInstallation.get()
+                && source->name() == QString::fromUtf8(flatpak_remote_get_name(remote.get()))) {
                 createPool(source);
-                if (source->remote() != remote) {
-                    m_refreshAppstreamMetadataJobs.remove(remote);
+                if (source->remote() != remote.get()) {
+                    m_refreshAppstreamMetadataJobs.remove(remote.get());
                 }
                 return source;
             }
@@ -1138,10 +1173,10 @@ QSharedPointer<FlatpakSource> FlatpakBackend::integrateRemote(FlatpakInstallatio
         return source;
     }
 
-    auto source = QSharedPointer<FlatpakSource>::create(this, flatpakInstallation, remote);
-    if (!source->isEnabled() || flatpak_remote_get_noenumerate(remote)) {
+    auto source = QSharedPointer<FlatpakSource>::create(this, flatpakInstallation.get(), remote.get());
+    if (!source->isEnabled() || flatpak_remote_get_noenumerate(remote.get())) {
         m_flatpakSources += source;
-        metadataRefreshed(remote);
+        metadataRefreshed(remote.get());
         return source;
     }
 
@@ -1150,6 +1185,17 @@ QSharedPointer<FlatpakSource> FlatpakBackend::integrateRemote(FlatpakInstallatio
     return source;
 }
 
+void FlatpakBackend::loadLocalUpdates()
+{
+    acquireFetching(true);
+    for (auto installation : std::as_const(m_installations)) {
+        if (g_cancellable_is_cancelled(m_cancellable)) {
+            break;
+        }
+        loadLocalUpdates(installation);
+    }
+    acquireFetching(false);
+}
 void FlatpakBackend::loadLocalUpdates(FlatpakInstallation *flatpakInstallation)
 {
     g_autoptr(GError) localError = nullptr;
@@ -1159,7 +1205,7 @@ void FlatpakBackend::loadLocalUpdates(FlatpakInstallation *flatpakInstallation)
         return;
     }
 
-    for (uint i = 0; i < refs->len; i++) {
+    for (uint i = 0; i < refs->len && !g_cancellable_is_cancelled(m_cancellable); i++) {
         FlatpakInstalledRef *ref = FLATPAK_INSTALLED_REF(g_ptr_array_index(refs, i));
 
         const gchar *latestCommit = flatpak_installed_ref_get_latest_commit(ref);
@@ -1271,7 +1317,7 @@ bool FlatpakBackend::updateAppMetadata(FlatpakResource *resource, const QString
 bool FlatpakBackend::updateAppMetadata(FlatpakResource *resource, const QByteArray &data)
 {
     // We just find the runtime with a regex, QSettings only can read from disk (and so does KConfig)
-    const QRegularExpression rx(QStringLiteral("runtime=(.*)"));
+    static const QRegularExpression rx(QStringLiteral("runtime=(.*)"));
     const auto match = rx.match(QString::fromUtf8(data));
     if (!match.isValid()) {
         return false;
@@ -1550,7 +1596,7 @@ void triage(FlatpakResource *resource,
 
 int FlatpakBackend::fetchingUpdatesProgress() const
 {
-    return m_updater->isSettingUp() ? 42 : 100;
+    return m_collector->progress();
 }
 
 ResultsStream *FlatpakBackend::search(const AbstractResourcesBackend::Filters &filter)
@@ -1588,7 +1634,7 @@ ResultsStream *FlatpakBackend::search(const AbstractResourcesBackend::Filters &f
                 const auto refBranch = flatpak_ref_get_branch(ref);
 
                 // need to issue the search of installed refs too
-                // it can happpen that a ref is present because it's installed but not part of the appstream metadata anymore
+                // it can happen that a ref is present because it's installed but not part of the appstream metadata anymore
                 for (auto installation : std::as_const(installations)) {
                     g_autoptr(GError) localError = nullptr;
                     g_autoptr(FlatpakInstalledRef) installedRef =
@@ -1600,7 +1646,7 @@ ResultsStream *FlatpakBackend::search(const AbstractResourcesBackend::Filters &f
                     }
                 }
                 if (filter.state < AbstractResource::Installed) {
-                    for (const auto &source : self->m_flatpakSources) {
+                    for (const auto &source : std::as_const(self->m_flatpakSources)) {
                         if (source->m_pool) {
                             auto components = source->componentsByFlatpakId(filter.resourceUrl.path());
                             resources.append(kTransform<QVector<StreamResult>>(components, [self, source](const AppStream::Component &comp) {
@@ -1780,7 +1826,7 @@ ResultsStream *FlatpakBackend::search(const AbstractResourcesBackend::Filters &f
                 auto *fw = new QFS(stream);
                 fw->setFuture(QtFuture::whenAll(futures.begin(), futures.end()));
                 FLATPAK_BACKEND_YIELD
-                for (auto r : unpooled) {
+                for (auto r : std::as_const(unpooled)) {
                     triage(r, prioritary, rest, filter, false);
                 }
                 connect(fw, &QFS::finished, stream, [self, futures, stream, _rest = rest, _prioritary = prioritary, filter, fw]() {
@@ -1874,24 +1920,6 @@ bool FlatpakBackend::isTracked(FlatpakResource *resource) const
     });
 }
 
-QVector<StreamResult> FlatpakBackend::resultsByAppstreamName(const QString &name) const
-{
-    QVector<StreamResult> resources;
-    for (const auto &source : m_flatpakSources) {
-        if (source->m_pool) {
-            auto components = source->componentsByName(name);
-            resources << kTransform<QVector<StreamResult>>(components, [this, source](const AppStream::Component &comp) -> StreamResult {
-                return {resourceForComponent(comp, source), comp.sortScore()};
-            });
-        }
-    }
-    auto f = [this](auto left, auto right) {
-        return flatpakResourceLessThan(left, right);
-    };
-    std::sort(resources.begin(), resources.end(), f);
-    return resources;
-}
-
 ResultsStream *FlatpakBackend::findResourceByPackageName(const QUrl &url)
 {
     if (url.scheme() == QLatin1String("appstream")) {
@@ -1901,21 +1929,44 @@ ResultsStream *FlatpakBackend::findResourceByPackageName(const QUrl &url)
         } else {
             auto stream = new ResultsStream(QStringLiteral("FlatpakStream-AppStreamUrl"));
             auto f = [this, stream, appstreamIds] {
-                std::set<AbstractResource *> resources;
-                QVector<StreamResult> resourcesVector;
-                for (const auto &appstreamId : appstreamIds) {
-                    const auto resourcesFound = resultsByAppstreamName(appstreamId);
-                    for (auto result : resourcesFound) {
-                        auto [x, inserted] = resources.insert(result.resource);
-                        if (inserted) {
-                            resourcesVector.append(result);
+                const auto pools = kTransform<QList<AppStream::ConcurrentPool *>>(m_flatpakSources, [](const auto &x) {
+                    return x->m_pool.get();
+                });
+                AppStream::ConcurrentPool::componentsByNames(&m_threadPool, pools, appstreamIds)
+                    .then(this, [this, stream](const QMap<AppStream::ConcurrentPool *, QList<AppStream::Component>> &componentsList) {
+                        QList<StreamResult> resourcesFound;
+                        std::set<AbstractResource *> resources;
+                        QVector<StreamResult> resourcesVector;
+                        auto sorter = [this](auto left, auto right) {
+                            return flatpakResourceLessThan(left, right);
+                        };
+
+                        for (const auto &[pool, components] : componentsList.asKeyValueRange()) {
+                            if (!pool) {
+                                continue;
+                            }
+                            const auto sourceIt = std::ranges::find_if(m_flatpakSources, [pool](const QSharedPointer<FlatpakSource> &s) -> bool {
+                                return s->m_pool.get() == pool;
+                            });
+                            Q_ASSERT(sourceIt != m_flatpakSources.end());
+                            const QSharedPointer<FlatpakSource> source = *sourceIt;
+                            resourcesFound << kTransform<QVector<StreamResult>>(components, [this, source](const AppStream::Component &comp) -> StreamResult {
+                                return {resourceForComponent(comp, source), comp.sortScore()};
+                            });
                         }
-                    }
-                }
-                if (!resourcesVector.isEmpty()) {
-                    Q_EMIT stream->resourcesFound(resourcesVector);
-                }
-                stream->finish();
+                        std::sort(resourcesFound.begin(), resourcesFound.end(), sorter);
+                        for (auto result : resourcesFound) {
+                            auto [x, inserted] = resources.insert(result.resource);
+                            if (inserted) {
+                                resourcesVector.append(result);
+                            }
+                        }
+
+                        if (!resourcesVector.isEmpty()) {
+                            Q_EMIT stream->resourcesFound(resourcesVector);
+                        }
+                        stream->finish();
+                    });
             };
 
             if (m_isFetching > 0) {
@@ -2089,7 +2140,10 @@ void FlatpakBackend::checkForUpdates()
     disconnect(this, &FlatpakBackend::initialized, m_checkForUpdatesTimer, qOverload<>(&QTimer::start));
     for (const auto &source : std::as_const(m_flatpakSources)) {
         if (source->remote()) {
-            Q_ASSERT(!m_refreshAppstreamMetadataJobs.contains(source->remote()));
+            if (m_refreshAppstreamMetadataJobs.contains(source->remote())) {
+                qCWarning(LIBDISCOVER_BACKEND_FLATPAK_LOG) << "Already checking for updates" << source->name();
+                continue;
+            }
             m_refreshAppstreamMetadataJobs.insert(source->remote());
             checkForRemoteUpdates(source->installation(), source->remote());
         }
@@ -2102,7 +2156,7 @@ void FlatpakBackend::checkForRemoteUpdates(FlatpakInstallation *installation, Fl
     const bool needsIntegration = m_refreshAppstreamMetadataJobs.contains(remote);
     if (flatpak_remote_get_disabled(remote) || flatpak_remote_get_noenumerate(remote)) {
         if (needsIntegration) {
-            integrateRemote(installation, remote);
+            integrateRemote(GLibHolder<FlatpakInstallation>(installation), GLibHolder<FlatpakRemote>(remote));
         }
         return;
     }
@@ -2115,6 +2169,7 @@ void FlatpakBackend::checkForRemoteUpdates(FlatpakInstallation *installation, Fl
         acquireFetching(false);
     });
 
+    m_collector->add(job);
     acquireFetching(true);
     job->start();
 }
