@@ -69,6 +69,12 @@ void StandardBackendUpdater::start()
         connect(t, &Transaction::downloadSpeedChanged, this, [this]() {
             Q_EMIT downloadSpeedChanged(downloadSpeed());
         });
+        connect(t, &Transaction::cancellableChanged, this, [this, t]() {
+            if (!m_canCancel && t->isCancellable()) {
+                m_canCancel = true;
+                Q_EMIT cancelableChanged(m_canCancel);
+            }
+        });
         connect(this, &StandardBackendUpdater::cancelTransaction, t, &Transaction::cancel);
         TransactionModel::global()->addTransaction(t);
         m_canCancel |= t->isCancellable();
@@ -149,7 +155,6 @@ void StandardBackendUpdater::transactionRemoved(Transaction *t)
             }
         }
     }
-    refreshUpdateable();
 }
 
 void StandardBackendUpdater::refreshProgress()
@@ -182,6 +187,7 @@ void StandardBackendUpdater::refreshUpdateable()
         return;
     }
 
+    m_timer.stop();
     setSettingUp(true);
     Q_EMIT progressingChanged(true);
     Q_EMIT fetchingChanged();
@@ -252,9 +258,9 @@ void StandardBackendUpdater::cleanup()
 {
     m_lastUpdate = QDateTime::currentDateTime();
     m_toUpgrade.clear();
+    Q_EMIT progressingChanged(false);
 
     refreshUpdateable();
-    Q_EMIT progressingChanged(false);
 }
 
 QList<AbstractResource *> StandardBackendUpdater::toUpdate() const
